#! /usr/bin/env python
# -*- coding: utf-8 -*-
# filename: tops_csv.py
# Copyright 2008, 2009, 2011 Stefano Costa <steko@iosa.it>
#
# This file is part of Total Open Station.
#
# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

import csv
import cStringIO


class OutputFormat:

    """
    Exports points data in CSV format.

    ``data`` should be an iterable (e.g. list) containing one iterable
    (e.g.  tuple) for each point. The default order is PID, x, x, z,
    TEXT.

    This is consistent with our current standard.
    """

    def __init__(self, data):
        self.data = data
        self.output = cStringIO.StringIO()
        self.writer = csv.writer(self.output, quoting=csv.QUOTE_NONNUMERIC)

    def process(self):
        self.writer.writerow(('PID', 'x', 'y', 'z', 'TEXT'))
        self.writer.writerows(self.data)
        return self.output.getvalue()

if __name__ == "__main__":
    TotalOpenCSV(
        [(1, 2, 3, 4, 'qwerty'),
         ("2.3", 42, 45, 12, 'asdfg')])



# -*- coding: utf-8 -*-
# filename: formats/leica_tcr_705.py
# Copyright 2009-2010 Stefano Costa <steko@iosa.it>
# Copyright 2009-2010 Luca Bianconi <luxetluc@yahoo.it>

# This file is part of Total Open Station.

# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

from . import Parser, Point


class FormatParser(Parser):

    def is_point(self, line):

        tokens = line.split(",")
        is_point = False

        try:
            float(tokens[1])
            float(tokens[2])
            float(tokens[3])
        except (ValueError, IndexError):
            is_point = False
        else:
            is_point = True
            x, y, z = tokens[1:4]

            # not so clear why there are such points recorded
            if x == '1.00' and y == '1.00' and z == '1.00':
                is_point = False

        return is_point

    def get_point(self, line):

        tokens = line.split(",")

        if len(tokens) > 4:
            text = str(tokens[4])
        else:
            text = ""

        p = Point(str(tokens[0]),
                  float(tokens[1]),
                  float(tokens[2]),
                  float(tokens[3]),
                  text)

        return p



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# filename: tops_dxf.py
# Copyright 2008-2009 Stefano Costa <steko@iosa.it>

# This file is part of Total Open Station.

# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.


class OutputFormat:

    """
    Exports points data in AutoCAD DXF format.

    It is based on the official DXF2000 documentation. Works with AutoCAD
    versions ranging at least from 2005 up to 2009, and QCAD.

    ``data`` should be an iterable (e.g. list) containing one iterable (e.g.
    tuple) for each point. The default order is PID, x, x, z, TEXT.

    This is consistent with our current standard.
    """

    def __init__(self, data, separate_layers=True):

        self.data = data
        self.separate_layers = separate_layers
        self.text_height = 0.05

    def process(self):
        '''Process the input data and return a string as output.

        This is because we want to keep separated the generation of output
        from saving it to disk.'''

        result = ''

        # extract layer list
        codes = set([p[4] for p in self.data])
        codes = [c.replace('.','_') for c in codes]
        layers = dict(enumerate(codes))
        colors = dict(zip(layers.values(), layers.keys()))

        # layer table
        result += '  0\nSECTION\n  2\nTABLES\n  0\nTABLE\n  2\nLAYER\n'
        for l in codes:
            if self.separate_layers is True:
                result += '  0\nLAYER\n'           # start definition of LAYER
                result += '  5\n10\n'              # LAYER handle
                result += '  2\n%s_PUNTI\n' % l    # LAYER name
                result += ' 70\n0\n'              # LAYER is not frozen
                result += ' 62\n%s\n' % (int(colors[l]) + 1) # LAYER color
                result += '  6\nCONTINUOUS\n'      # LAYER linetype

                result += '  0\nLAYER\n'           # same as above
                result += '  5\n10\n'
                result += '  2\n%s_QUOTE\n' % l
                result += ' 70\n0\n'
                result += ' 62\n%s\n' % (int(colors[l]) + 1)
                result += '  6\nCONTINUOUS\n'

                result += '  0\nLAYER\n'           # ditto
                result += '  5\n10\n'
                result += '  2\n%s_NUMERI\n' % l
                result += ' 70\n0\n'
                result += ' 62\n%s\n' % (int(colors[l]) + 1)
                result += '  6\nCONTINUOUS\n'
            else:
                result += '  0\nLAYER\n'           # ditto
                result += '  5\n10\n'
                result += '  2\n%s\n' % l          # LAYER name w/o any suffix
                result += ' 70\n0\n'
                result += ' 62\n%s\n' % (int(colors[l]) + 1)
                result += '  6\nCONTINUOUS\n'

        result += '  0\nENDTAB\n  0\nENDSEC\n'

        # drawing entities
        result += '  0\nSECTION\n  2\nENTITIES\n'

        for p in self.data:
            p_id, p_x, p_y, p_z, p_layer = p
            if self.separate_layers is True:
                layer_point = "%s_PUNTI" % p_layer
                layer_z_text = "%s_QUOTE" % p_layer
                layer_id_text = "%s_NUMERI" % p_layer
            else:
                layer_point = layer_z_text = layer_id_text = p_layer
            p_yz = str(float(p_y) - (self.text_height * 1.2))

            # add point
            result += '  0\nPOINT\n'
            result += '  8\n%s\n' % layer_point
            result += ' 10\n%s\n' % p_x
            result += ' 20\n%s\n' % p_y

            # add ID number
            result += '  0\nTEXT\n'
            result += '  1\n%s\n' % p_id
            result += '  8\n%s\n' % layer_id_text
            result += ' 10\n%s\n' % p_x
            result += ' 20\n%s\n' % p_y
            result += ' 40\n%01.2f\n' % self.text_height
            result += ' 62\n256\n'

            # add Z value as string
            result += '  0\nTEXT\n'
            result += '  1\n%s\n' % p_z
            result += '  8\n%s\n' % layer_z_text
            result += ' 10\n%s\n' % p_x
            result += ' 20\n%s\n' % p_yz
            result += ' 40\n%01.2f\n' % self.text_height
            result += ' 62\n256\n'

        result += '  0\nENDSEC\n  0\nEOF\n'
        return result


if __name__ == "__main__":
    TotalOpenDXF(
        [(1, 2, 3, 4, 'qwerty'),
         ("2.3", 42, 45, 12, 'asdfg')])



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# filename: trimble.py
# Copyright 2009 Luca Bianconi <luxetluc@yahoo.it>
# Copyright 2009,2011 Stefano Costa <steko@iosa.it>
# Copyright 2009 Alessandro Bezzi <alessandro.bezzi@arc-team.com>
# Under the GNU GPL 3 License

from . import *


class ModelConnector(Connector):

    """Trimble Geodimeter 600"""

    def __init__(self, port):
        Connector.__init__(
            self,
            port=port,
            baudrate=9600,
            bytesize=8,
            stopbits=1,
            parity='N')



#! /usr/bin/env python

__all__ = ["tops_csv", "tops_dxf", "tops_dat", "tops_sql", "tops_txt"]

BUILTIN_OUTPUT_FORMATS = {
    'dxf': ('tops_dxf', 'OutputFormat', 'DXF'),
    'csv': ('tops_csv', 'OutputFormat', 'CSV'),
    'sql': ('tops_sql', 'OutputFormat', 'OGC-SQL'),
    'dat': ('tops_dat', 'OutputFormat', 'DAT'),
    'txt': ('tops_txt', 'OutputFormat', 'Text'),
    }

# Conditional formats
try:
    import geojson
except ImportError:
    pass
else:
    BUILTIN_OUTPUT_FORMATS['geojson'] = ('tops_geojson', 'OutputFormat', 'GeoJSON')



# -*- coding: utf-8 -*-
# filename: formats/__init__.py
# Copyright 2008-2010 Stefano Costa <steko@iosa.it>
# Copyright 2008 Luca Bianconi <luxetluc@yahoo.it>

# This file is part of Total Open Station.

# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.


class Point:

    def __init__(self, p_id, x, y, z, text):

        self.p_id = p_id
        self.x = x
        self.y = y
        self.z = z
        self.text = text

        self.tuplepoint = (self.p_id, self.x, self.y, self.z, self.text)

    def __str__(self):
        return 'Point(%(p_id)s, %(x)s, %(y)s, %(z)s, %(text)s)' % self.__dict__


class Parser:
    '''Parses a *single* string of raw data.

    This means that if you plan to load data from a file you have to
    pass the output of open(file).read() to this class.'''

    def __init__(self, data, swapXY=False):

        self.data = data
        self.d = self.split_points()
        self.swapXY = swapXY

        valid_lines = filter(self.is_point, self.d)
        fg_lines = map(self.get_point, valid_lines)
        self.points = [p.tuplepoint for p in fg_lines if p is not None]

    def is_point(self, point):
        """Action for finding which parts of the source file are points.

        This method **must** be overridden in the child class
        to have a working parser."""

        pass

    def get_point(self, point):
        """Action for getting points from source file.

        This method **must** be overridden in the child class
        to have a working parser."""

        pass

    def split_points(self):
        """Action for splitting points.

        Defaults to ``splitlines()`` because most formats have one
        point per line.

        Override this method if the format is different."""

        return self.data.splitlines()


BUILTIN_INPUT_FORMATS = {
    'leica_tcr_1205': ('leica_tcr_1205', 'FormatParser', 'Leica TCR 1205'),
    'zeiss_rec_500': ('zeiss_rec_500', 'FormatParser', 'Zeiss REC 500'),
    'nikon_raw_v200': ('nikon_raw_v200', 'FormatParser','Nikon RAW V2.00'),
    'leica_tcr_705': ('leica_tcr_705', 'FormatParser', 'Leica TCR 705'),
    'trimble_are': ('trimble_are', 'FormatParser', 'Trimble AREA'),
    'topcon_gts': ('topcon_gts', 'FormatParser', 'Topcon GTS'),
    }



# -*- coding: utf-8 -*-
# filename: polar.py
# Copyright 2010 Stefano Costa <steko@iosa.it>

# This file is part of Total Open Station.

# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

import math

from . import Point


def polar_to_cartesian(base_x, base_y, base_z, dist, angle, z_angle, ih, th):
    '''Convert polar coordinates to cartesian.

    Needs both base point coordinates and measurement angle and distance.
    Angles should be given in radiants.'''

    rad_angle = angle
    rad_z_angle = z_angle
    dist_r = math.sin(rad_z_angle) * dist
    target_x = base_x + math.cos(rad_angle) * dist_r
    target_y = base_y + math.sin(rad_angle) * dist_r
    target_z = base_z + math.cos(rad_z_angle) * dist
    target_z = target_z + ih - th
    return dict(x=target_x, y=target_y, z=target_z)


class PolarPoint:
    '''A point geometry defined by polar coordinates.'''

    def __init__(self,
                 dist,                # inclined distance
                 angle,               # horizontal angle
                 z_angle,             # vertical angle
                 th,                  # target height
                 angle_type,          # degrees or gons
                 base_point,          # BasePoint object
                 pid,                 # point ID
                 text,                # point description
                 coordorder='NEZ'):   # cartesian coordinates order (NEZ, ENZ)
        self.dist = float(dist)
        self.th = float(th)
        self.angle_type = angle_type
        if angle_type is 'deg':
            self.angle = math.radians(float(angle))
            self.z_angle = math.radians(float(z_angle))
        if angle_type is 'gon':
            self.angle = math.radians(float(angle)*0.9)
            self.z_angle = math.radians(float(z_angle)*0.9)
        self.pid = pid
        self.text = text
        self.coordorder = coordorder
        # base point data
        self.base_x = base_point.x
        self.base_y = base_point.y
        self.base_z = base_point.z
        self.ih = base_point.ih

    def to_cartesian(self):
        '''Converts from polar to cartesian.'''

        coords = polar_to_cartesian(self.base_x,
                                    self.base_y,
                                    self.base_z,
                                    self.dist,
                                    self.angle,
                                    self.z_angle,
                                    self.ih,
                                    self.th)
        return "%(x)s,%(y)s,%(z)s" % coords

    def to_point(self):
        '''Convert from PolarPoint to (cartesian) Point object'''

        cart_coords = polar_to_cartesian(self.base_x,
                                    self.base_y,
                                    self.base_z,
                                    self.dist,
                                    self.angle,
                                    self.z_angle,
                                    self.ih,
                                    self.th)

        if self.coordorder == 'NEZ':
            cart_coords['x'], cart_coords['y'] = cart_coords['y'], cart_coords['x']

        cart_point = Point(self.pid,
                           cart_coords['x'],
                           cart_coords['y'],
                           cart_coords['z'],
                           self.text)

        return cart_point.tuplepoint


class BasePoint:
    '''A base point to derive cartesian coordinates from polar.

    TODO: find out whether ih is more commonly coupled to a base point or
    to each single point.'''

    def __init__(self, x, y, z, ih):
        self.x = float(x)
        self.y = float(y)
        self.z = float(z)
        self.ih = float(ih)



# -*- coding: utf-8 -*-

# Copyright 2007-2011 by the Sphinx team http://sphinx.pocoo.org/

from os import path

__version__  = '0.3'

package_dir = path.abspath(path.dirname(__file__))

if '+' in __version__ or 'pre' in __version__:
    # try to find out the changeset hash if checked out from hg, and append
    # it to __version__ (since we use this value from setup.py, it gets
    # automatically propagated to an installed copy as well)
    try:
        import subprocess
        p = subprocess.Popen(['hg', 'id', '-i', '-R',
                              path.join(package_dir, '..')],
                             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out, err = p.communicate()
        if out:
            __version__ += '/' + out.strip()
    except Exception:
        pass



# -*- coding: utf-8 -*-
# filename: formats/nikon_raw_v200.py
# Copyright 2010 Stefano Costa <steko@iosa.it>
#
# This file is part of Total Open Station.
#
# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

from polar import BasePoint, PolarPoint


class FormatParser:
    '''A FormatParser for Nikon RAW data format V2.00.

    It doesn't inherit from the base Parser class because the internal
    procedure is quite different, but it implements the same API so it
    can work nicely with other parts of the library.'''

    def __init__(self, data):
        self.rows = data.splitlines()

    def _points(self):
        points = []
        for row in self.rows:
            fs = row.split(',')
            if fs[0] == 'CO' and fs[1].startswith('Coord Order:'):
                coordorder = fs[1].split(':')[-1].strip()
            if fs[0] == 'ST':
                x = fs[6]   # FIXME NEZ coord order shouldn't be hardcoded
                y = fs[7]
                z = fs[5]
                bp = BasePoint(x=x, y=y, z=z, ih=0)
            if fs[0] == 'SS':
                angle = fs[4]
                z_angle = fs[5]
                dist = fs[3]
                th = fs[2]
                pid = fs[1]
                try:
                    text = fs[7]
                except IndexError:
                    text = fs[-1]
                p = PolarPoint(dist=dist,
                               angle=angle,
                               z_angle=z_angle,
                               th=th,
                               angle_type='gon',
                               base_point=bp,
                               pid=pid,
                               text=text,
                               coordorder=coordorder)
                points.append(p.to_point())
        return points

    points = property(_points)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# filename: upref.py
# Copyright 2010 Stefano Costa <steko@iosa.it>
# Copyright 2010 Luca Bianconi <luxetluc@yahoo.it>
#
# This file is part of Total Open Station.
#
# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

import atexit
import os
import os.path

from ConfigParser import ConfigParser, NoSectionError


class UserPrefs(ConfigParser):
    '''Manage user preferences for GUI options and last used values.

    Proof-of-concept. Notes:

    * user preferences path is hardcoded here
    * returns a dictionary of user preferences
    * preferences will be set through a dictionary as well'''


    def __init__(self):

        ConfigParser.__init__(self)

        USER_PREFS_PATH = '~/.totalopenstation/totalopenstation.cfg'

        self.upref = os.path.expanduser(USER_PREFS_PATH)

        if os.path.exists(self.upref):
            self.read(self.upref)
            try:
                self.getvalue('model')
            except NoSectionError:
                self.initfile()
        elif not os.path.exists(os.path.dirname(self.upref)):
            os.mkdir(os.path.dirname(self.upref))
            self.initfile()
        else:
            self.initfile()

    def initfile(self):
        self.write()
        print('User preferences do not exist!')
        self.add_section('topsconfig')
        self.set('topsconfig', 'model','')
        self.set('topsconfig', 'port', '')
        print('Created new user preferences file')

    def write(self):
        ''' override ConfigParser.write() method '''

        ConfigParser.write(self, open(self.upref, 'w'))

    def getdict(self):
        ''' get config file values '''

        model = self.get('topsconfig', 'model')
        port = self.get('topsconfig', 'port')

        return {'model':model , 'port': port}

    def getvalue(self, value):
        ''' get specific config file value '''

        return self.get('topsconfig', value)

    def setvalues(self, values):
        ''' set specific config file value '''

        self.set('topsconfig', 'model', values['model'])
        self.set('topsconfig', 'port', values['port'])

        self.write()



# -*- coding: utf-8 -*-
# filename: tops_geojson.py
# Copyright 2009 Stefano Costa <steko@iosa.it>

# This file is part of Total Open Station.

# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

import geojson


def output_geojson(data):
    '''A GeoJSON output driver.'''

    fs = []

    for point in data:
        pid, x, y, z, text = point
        x, y, z = [float(c) for c in x, y, z]
        p = geojson.Point([x, y, z])
        prop = dict(text=text)
        f = geojson.Feature(id=pid, geometry=p, properties=prop)
        fs.append(f)
    fc = geojson.FeatureCollection(fs)
    return geojson.dumps(fc)


class TotalOpenGEOJSON:
    '''A GeoJSON output driver.

    Depends on the geojson package http://pypi.python.org/pypi/geojson/

    FIXME This is an example of how classes are probably useless for
    coding output formats. A transition towards an eventual new
    structure could be done in 2 passages, the first being making the
    current classes empty wrappers around a function.'''

    def __init__(self, data):

        self.data = data

    def process(self):

        return output_geojson(self.data)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# filename: leica_tcr_1205.py
# Copyright 2008 Luca Bianconi <luxetluc@yahoo.it>
# Copyright 2008,2011 Stefano Costa <steko@iosa.it>
# Under the GNU GPL 3 License

from . import *


class ModelConnector(Connector):

    def __init__(self, port):
        Connector.__init__(self, port=port, baudrate=19200)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# filename: tops_sql.py
# Copyright 2008-2010 Stefano Costa <steko@iosa.it>
#
# This file is part of Total Open Station.
#
# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.


def to_sql(point, tablename):
    '''Generate SQL line corresponding to the input point.

    At this moment the column names are fixed, but they could change in the
    future. The default names are reasonable.'''

    params = {
        'wkt': to_wkt(point),
        'tablename': tablename,
        'pid': point[0],
        'text': point[4]}
    sql_string = "INSERT INTO %(tablename)s" % params
    sql_string += "(point_id, point_geom, point_text) VALUES"
    sql_string += "(%(pid)s,GeomFromText('%(wkt)s'),'%(text)s');\n" % params
    return sql_string


def to_wkt(point):
    pid, x, y, z, text = point
    wkt_representation = 'POINT(%s %s)' % (x, y)
    return wkt_representation


class OutputFormat:

    """
    Exports points data in SQL format suitable for use with PostGIS & friends.

    http://postgis.refractions.net/documentation/manual-1.3/ch04.html#id2986280
    has an example of loading an SQL file into a PostgreSQL database.

    ``data`` should be an iterable (e.g. list) containing one iterable (e.g.
    tuple) for each point. The default order is PID, x, x, z, TEXT.

    This is consistent with our current standard.
    """

    def __init__(self, data, tablename='topsdata'):
        self.data = data
        self.tablename = tablename

    def process(self):
        lines = [to_sql(e, self.tablename) for e in self.data]
        lines.insert(0, 'BEGIN;\n')
        lines.append('COMMIT;\n')
        output = "".join(lines)
        return output

if __name__ == "__main__":
    TotalOpenSQL(
        [(1, 2, 3, 4, 'qwerty'),
         ("2.3", 42, 45, 12, 'asdfg')],
        'prova')



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# filename: nikon_npl_350.py
# Copyright 2008,2011 Stefano Costa <steko@iosa.it>
# Under the GNU GPL 3 License

from . import *


class ModelConnector(Connector):

    def __init__(self, port):
        Connector.__init__(self, port=port, baudrate=1200, xonxoff=True)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# filename: totalopenstation-cli-parser.py
# Copyright 2008-2009 Stefano Costa <steko@iosa.it>

# This file is part of Total Open Station.

# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

import sys
import os
import gettext

from optparse import OptionParser

from totalopenstation.formats import BUILTIN_INPUT_FORMATS
from totalopenstation.output import BUILTIN_OUTPUT_FORMATS


t = gettext.translation('totalopenstation', './locale', fallback=True)
_ = t.lgettext

usage = _("usage: %prog [option] arg1 [option] arg2 ...")

parser = OptionParser(usage=usage)
parser.add_option("-i",
                "--infile",
                action="store",
                type="string",
                dest="infile",
                help=_("select input FILE  (do not specify for stdin)"),
                metavar="FILE")
parser.add_option("-o",
                "--outfile",
                action="store",
                type="string",
                dest="outfile",
                help=_("select output FILE (do not specify for stdout)"),
                metavar="FILE")
parser.add_option("-f",
                "--input-format",
                action="store",
                type="string",
                dest="informat",
                help=_("select input FORMAT"),
                metavar="FORMAT")
parser.add_option("-t",
                "--output-format",
                action="store",
                type="string",
                dest="outformat",
                help=_("select input FORMAT"),
                metavar="FORMAT")
parser.add_option(
                "--overwrite",
                action="store_true",
                dest="overwrite",
                default=False,
                help=_("overwrite existing output file"))
parser.add_option(
    "--list",
    action="store_true",
    dest="list",
    default=False,
    help=_("list the available input and output formats"))


(options, args) = parser.parse_args()

def list_formats():
    '''Print a list of the supported input and output formats.'''

    from totalopenstation.formats import BUILTIN_INPUT_FORMATS

    mod_string = "List of supported input formats:\n" + "-" * 30 + "\n"
    for k, v in sorted(BUILTIN_INPUT_FORMATS.items()):
        mod_string += k.ljust(20) + v[2] + "\n"
    mod_string += "\n\n"

    mod_string += "List of supported output formats:\n" + "-" * 30 + "\n"
    for k, v in sorted(BUILTIN_OUTPUT_FORMATS.items()):
        mod_string += k.ljust(20) + v[2] + "\n"
    mod_string += "\n"
    return mod_string

if options.list:
    sys.stdout.write(list_formats())
    sys.exit()

if options.informat:
    inputclass = BUILTIN_INPUT_FORMATS[options.informat]
    if isinstance(inputclass, tuple):
        try:
            # builtin format parser
            mod, cls, name = inputclass
            inputclass = getattr(
                __import__('totalopenstation.formats.' + mod, None, None, [cls]), cls)
        except ImportError, message:
            sys.exit(_("\nError:\n%s\n\n%s") % (message, list_formats()))
else:
    sys.exit(_("Please specify an input format"))

if options.outformat:
    outputclass = BUILTIN_OUTPUT_FORMATS[options.outformat]
    if isinstance(outputclass, tuple):
        try:
            # builtin output builder
            mod, cls, name = outputclass
            outputclass = getattr(
                __import__('totalopenstation.output.' + mod, None, None, [cls]), cls)
        except ImportError, message:
            sys.exit(_("\nError:\n%s\n\n%s") % (message, list_formats()))

if options.infile:
    infile = open(options.infile, 'r').read()
else:
    infile = sys.stdin.read()


def main(infile):
    '''After setting up all parameters, finally try to process input data.'''

    parsed_data = inputclass(infile)
    parsed_points = parsed_data.points
    output = outputclass(parsed_points)

    def write_to_file(outfile):
        e = open(outfile, 'w')
        e.write(output.process())
        e.close()

    if options.outfile:
        if not os.path.exists(options.outfile):
            write_to_file(options.outfile)
            print _("Downloaded data saved to out file %s") % options.outfile
        else:
            if options.overwrite:
                write_to_file(options.outfile)
                print _("Downloaded data saved to file %s,") % options.outfile,
                print _("overwriting the existing file")
            else:
                sys.exit(_("Specified output file already exists\n"))
    else:
        sys.stdout.write(output.process())

if __name__ == '__main__':
    main(infile)
